@@ -10,6 +10,7 @@ use serde_amqp::primitives::Symbol;
 use tokio::{
     io::{AsyncRead, AsyncWrite},
     net::TcpStream,
+    sync::mpsc::{self},
 };
 use url::Url;
 
@@ -17,8 +18,8 @@ use crate::{
     connection::{Connection, ConnectionState},
     frames::amqp,
     sasl_profile::SaslProfile,
-    transport::protocol_header::ProtocolHeader,
     transport::Transport,
+    transport::{error::NegotiationError, protocol_header::ProtocolHeader},
 };
 
 use super::{
@@ -29,15 +30,27 @@ use super::{
 pub(crate) const DEFAULT_CONTROL_CHAN_BUF: usize = 128;
 pub(crate) const DEFAULT_OUTGOING_BUFFER_SIZE: usize = u16::MAX as usize;
 
-/// Type state for connection [`Builder`] representing state where a valid container id is not present
-#[derive(Debug)]
-pub struct WithoutContainerId {}
+pub mod mode {
+    //! Type states for [`crate::connection::Builder`]
 
-/// Type state for connection [`Builder`] representing state where a valid container id is present
-#[derive(Debug)]
-pub struct WithContainerId {}
+    // /// Type state for [`crate::connection::Builder`] representing state where a valid container id is not present
+    // #[derive(Debug)]
+    // pub struct WithoutContainerId {}
+
+    // /// Type state for [`crate::connection::Builder`] representing state where a valid container id is present
+    // #[derive(Debug)]
+    // pub struct WithContainerId {}
+
+    /// Type state for [`crate::connection::Builder`]
+    #[derive(Debug)]
+    pub struct ConnectorWithId {}
+    /// Type state for [`crate::connection::Builder`]
+    #[derive(Debug)]
+    pub struct ConnectorNoId {}
+}
 
 /// Builder for [`crate::Connection`]
+#[derive(Clone)]
 pub struct Builder<'a, Mode, Tls> {
     /// The id of the source container
     pub container_id: String,
@@ -79,7 +92,13 @@ pub struct Builder<'a, Mode, Tls> {
     /// Connection properties
     pub properties: Option<Fields>,
 
-    /// TLS client config
+    /// TLS connector.
+    ///
+    /// If `"rustls"` is enabled, this field will be `tokio_rustls::TlsConnector`.
+    ///
+    /// If `"native-tls"` is enabled, this field will be `tokio_native_tls::TlsConnector`.
+    ///
+    /// If none of the above conditions were true, this will default to unit type `()`.
     pub tls_connector: Tls,
 
     /// Buffer size of the underlying [`tokio::sync::mpsc::channel`] that are used by the sessions
@@ -103,6 +122,24 @@ pub struct Builder<'a, Mode, Tls> {
     marker: PhantomData<Mode>,
 }
 
+impl<'a, Tls> From<Builder<'a, mode::ConnectorWithId, Tls>> for Open {
+    fn from(builder: Builder<'a, mode::ConnectorWithId, Tls>) -> Self {
+        Open {
+            container_id: builder.container_id,
+            hostname: builder.hostname.map(Into::into),
+            max_frame_size: builder.max_frame_size,
+            channel_max: builder.channel_max,
+            // To avoid spurious timeouts, the value in idle-time-out SHOULD be half the peer’s actual timeout threshold.
+            idle_time_out: builder.idle_time_out.map(|v| v / 2),
+            outgoing_locales: builder.outgoing_locales,
+            incoming_locales: builder.incoming_locales,
+            offered_capabilities: builder.offered_capabilities,
+            desired_capabilities: builder.desired_capabilities,
+            properties: builder.properties,
+        }
+    }
+}
+
 impl<'a, Mode: std::fmt::Debug> std::fmt::Debug for Builder<'a, Mode, ()> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("Builder")
@@ -151,7 +188,9 @@ impl<'a, Mode: std::fmt::Debug> std::fmt::Debug for Builder<'a, Mode, tokio_rust
 }
 
 #[cfg(feature = "native-tls")]
-impl<'a, Mode: std::fmt::Debug> std::fmt::Debug for Builder<'a, Mode, tokio_native_tls::TlsConnector> {
+impl<'a, Mode: std::fmt::Debug> std::fmt::Debug
+    for Builder<'a, Mode, tokio_native_tls::TlsConnector>
+{
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("Builder")
             .field("container_id", &self.container_id)
@@ -174,7 +213,7 @@ impl<'a, Mode: std::fmt::Debug> std::fmt::Debug for Builder<'a, Mode, tokio_nati
     }
 }
 
-impl<'a> Builder<'a, WithoutContainerId, ()> {
+impl<'a, Mode> Builder<'a, Mode, ()> {
     /// Creates a new builder for [`crate::Connection`]
     pub fn new() -> Self {
         Self {
@@ -202,9 +241,9 @@ impl<'a> Builder<'a, WithoutContainerId, ()> {
     }
 }
 
-impl<'a, Mode, Tls> Builder<'a, Mode, Tls> {
+impl<'a, Tls> Builder<'a, mode::ConnectorNoId, Tls> {
     /// The id of the source container
-    pub fn container_id(self, id: impl Into<String>) -> Builder<'a, WithContainerId, Tls> {
+    pub fn container_id(self, id: impl Into<String>) -> Builder<'a, mode::ConnectorWithId, Tls> {
         // In Rust, it’s more common to pass slices as arguments
         // rather than vectors when you just want to provide read access.
         // The same goes for String and &str.
@@ -230,10 +269,28 @@ impl<'a, Mode, Tls> Builder<'a, Mode, Tls> {
             marker: PhantomData,
         }
     }
+}
+
+impl<'a, Mode, Tls> Builder<'a, Mode, Tls> {
+    /// Alias for [`rustls_connector`](#method.rustls_connector) if only `"rustls"` is enabled
+    #[cfg_attr(docsrs, doc(cfg(all(feature = "rustls", not(feature = "native-tls")))))]
+    #[cfg(any(docsrs, all(feature = "rustls", not(feature = "native-tls"))))]
+    pub fn tls_connector(
+        self,
+        tls_connector: tokio_rustls::TlsConnector,
+    ) -> Builder<'a, Mode, tokio_rustls::TlsConnector> {
+        self.rustls_connector(tls_connector)
+    }
 
     /// Set the TLS connector with `tokio-rustls`
-    #[cfg(all(feature = "rustls", not(feature = "native-tls")))]
-    pub fn tls_connector(self, tls_connector: tokio_rustls::TlsConnector) -> Builder<'a, Mode, tokio_rustls::TlsConnector> {
+    ///
+    /// If only one of `"rustls"` or `"native-tls"` is enabled, a convenience alias function `tls_connector()` is provided.
+    #[cfg_attr(docsrs, doc(cfg(feature = "rustls")))]
+    #[cfg(all(feature = "rustls"))]
+    pub fn rustls_connector(
+        self,
+        tls_connector: tokio_rustls::TlsConnector,
+    ) -> Builder<'a, Mode, tokio_rustls::TlsConnector> {
         // In Rust, it’s more common to pass slices as arguments
         // rather than vectors when you just want to provide read access.
         // The same goes for String and &str.
@@ -260,9 +317,26 @@ impl<'a, Mode, Tls> Builder<'a, Mode, Tls> {
         }
     }
 
+    /// Alias for [`native_tls_connector`](#method.native_tls_connector) if only `"native-tls"` is
+    /// enabled.
+    #[cfg_attr(docsrs, doc(cfg(all(feature = "native-tls", not(feature = "rustls")))))]
+    #[cfg(any(docsrs, all(feature = "native-tls", not(feature = "rustls"))))]
+    pub fn tls_connector(
+        self,
+        tls_connector: tokio_native_tls::TlsConnector,
+    ) -> Builder<'a, Mode, tokio_native_tls::TlsConnector> {
+        self.native_tls_connector(tls_connector)
+    }
+
     /// Set the TLS connector with `tokio-native-tls`
-    #[cfg(all(feature = "native-tls", not(feature = "rustls")))]
-    pub fn tls_connector(self, tls_connector: tokio_native_tls::TlsConnector) -> Builder<'a, Mode, tokio_native_tls::TlsConnector> {
+    ///
+    /// If only one of `"rustls"` or `"native-tls"` is enabled, a convenience alias function `tls_connector()` is provided.
+    #[cfg_attr(docsrs, doc(cfg(feature = "native-tls")))]
+    #[cfg(feature = "native-tls")]
+    pub fn native_tls_connector(
+        self,
+        tls_connector: tokio_native_tls::TlsConnector,
+    ) -> Builder<'a, Mode, tokio_native_tls::TlsConnector> {
         // In Rust, it’s more common to pass slices as arguments
         // rather than vectors when you just want to provide read access.
         // The same goes for String and &str.
@@ -428,8 +502,32 @@ impl<'a, Mode> Builder<'a, Mode, ()> {
     }
 }
 
-impl<'a, Tls> Builder<'a, WithContainerId, Tls> {
-    async fn connect_with_stream<Io>(mut self, stream: Io) -> Result<ConnectionHandle, OpenError>
+impl<'a, Tls> Builder<'a, mode::ConnectorWithId, Tls> {
+    #[inline]
+    async fn negotiate_amqp_with_stream<Io>(
+        &self,
+        mut stream: Io,
+        local_state: &mut ConnectionState,
+    ) -> Result<Transport<Io, amqp::Frame>, NegotiationError>
+    where
+        Io: AsyncRead + AsyncWrite + std::fmt::Debug + Send + Unpin + 'static,
+    {
+        let _remote_header =
+            Transport::negotiate(&mut stream, local_state, ProtocolHeader::amqp()).await?;
+        let idle_timeout = self
+            .idle_time_out
+            .map(|millis| Duration::from_millis(millis as u64));
+        // Prior to any explicit negotiation, the maximum frame size is 512 (MIN-MAX-FRAME-SIZE) and the maximum
+        // channel number is 0
+        let transport =
+            Transport::<Io, amqp::Frame>::bind(stream, MIN_MAX_FRAME_SIZE, idle_timeout);
+        Ok(transport)
+    }
+
+    async fn connect_with_stream<Io>(
+        mut self,
+        stream: Io,
+    ) -> Result<ConnectionHandle<()>, OpenError>
     where
         Io: AsyncRead + AsyncWrite + std::fmt::Debug + Send + Unpin + 'static,
     {
@@ -437,58 +535,38 @@ impl<'a, Tls> Builder<'a, WithContainerId, Tls> {
             Some(profile) => {
                 let hostname = self.hostname;
                 let stream = Transport::connect_sasl(stream, hostname, profile).await?;
-                self.connect_with_stream_inner(stream).await
-            }
-            None => {
-                self.connect_with_stream_inner(stream).await
+                self.connect_amqp_with_stream(stream).await
             }
+            None => self.connect_amqp_with_stream(stream).await,
         }
     }
 
-    async fn connect_with_stream_inner<Io>(
+    async fn connect_amqp_with_stream<Io>(
         self,
-        mut stream: Io,
-    ) -> Result<ConnectionHandle, OpenError>
+        stream: Io,
+    ) -> Result<ConnectionHandle<()>, OpenError>
     where
         Io: AsyncRead + AsyncWrite + std::fmt::Debug + Send + Unpin + 'static,
     {
-        use tokio::sync::mpsc;
-
         // exchange header
         let mut local_state = ConnectionState::Start;
-        let _remote_header =
-            Transport::negotiate(&mut stream, &mut local_state, ProtocolHeader::amqp()).await?;
-        let idle_timeout = self
-            .idle_time_out
-            .map(|millis| Duration::from_millis(millis as u64));
-        let transport =
-            Transport::<_, amqp::Frame>::bind(stream, self.max_frame_size.0 as usize, idle_timeout);
+        let buffer_size = self.buffer_size;
+        let transport = self
+            .negotiate_amqp_with_stream(stream, &mut local_state)
+            .await?;
 
         // spawn Connection Mux
-        let local_open = Open {
-            container_id: self.container_id,
-            hostname: self.hostname.map(Into::into),
-            max_frame_size: self.max_frame_size,
-            channel_max: self.channel_max,
-            // To avoid spurious timeouts, the value in idle-time-out SHOULD be half the peer’s actual timeout threshold.
-            idle_time_out: self.idle_time_out.map(|v| v / 2),
-            outgoing_locales: self.outgoing_locales,
-            incoming_locales: self.incoming_locales,
-            offered_capabilities: self.offered_capabilities,
-            desired_capabilities: self.desired_capabilities,
-            properties: self.properties,
-        };
+        let local_open = Open::from(self);
 
         // create channels
-        let (connection_control_tx, connection_control_rx) =
-            mpsc::channel(DEFAULT_CONTROL_CHAN_BUF);
-        let (outgoing_tx, outgoing_rx) = mpsc::channel(self.buffer_size);
+        let (control_tx, control_rx) = mpsc::channel(DEFAULT_CONTROL_CHAN_BUF);
+        let (outgoing_tx, outgoing_rx) = mpsc::channel(buffer_size);
+        let connection = Connection::new(control_tx.clone(), local_state, local_open);
 
-        let connection = Connection::new(connection_control_tx.clone(), local_state, local_open);
         let engine = ConnectionEngine::open(
             transport,
             connection,
-            connection_control_rx,
+            control_rx,
             outgoing_rx,
             // session_control_rx
         )
@@ -496,16 +574,17 @@ impl<'a, Tls> Builder<'a, WithContainerId, Tls> {
         let handle = engine.spawn();
 
         let connection_handle = ConnectionHandle {
-            control: connection_control_tx,
+            control: control_tx,
             handle,
             outgoing: outgoing_tx, // session_control: session_control_tx
+            session_listener: (),
         };
 
         Ok(connection_handle)
     }
 }
 
-impl<'a> Builder<'a, WithContainerId, ()> {
+impl<'a> Builder<'a, mode::ConnectorWithId, ()> {
     /// Open a [`crate::Connection`] with an url
     ///
     /// # Raw AMQP connection
@@ -520,14 +599,14 @@ impl<'a> Builder<'a, WithContainerId, ()> {
     /// # TLS
     ///
     /// TLS is not supported unless one and only one of the following feature must be enabled
-    /// 
+    ///
     /// 1. "rustls"
     /// 2. "native-tls"
-    /// 
+    ///
     /// If no custom `TlsConnector` is supplied, the following default connector will be used.
-    /// 
+    ///
     /// ## Default TLS connector with `"rustls"` enabled
-    /// 
+    ///
     /// ```rust,ignore
     /// let mut root_cert_store = RootCertStore::empty();
     /// root_cert_store.add_server_trust_anchors(webpki_roots::TLS_SERVER_ROOTS.0.iter().map(
@@ -542,17 +621,17 @@ impl<'a> Builder<'a, WithContainerId, ()> {
     /// let config = ClientConfig::builder()
     ///     .with_safe_defaults()
     ///     .with_root_certificates(root_cert_store)
-    ///     .with_no_client_auth(); 
+    ///     .with_no_client_auth();
     /// let connector = TlsConnector::from(Arc::new(config));
     /// ```
-    /// 
+    ///
     /// ## Default TLS connector with `"native-tls"` enabled
-    /// 
+    ///
     /// ```rust,ignore
     /// let connector = native_tls::TlsConnector::new().unwrap();
     /// let connector = tokio_native_tls::TlsConnector::from(connector);
     /// ```
-    /// 
+    ///
     /// # SASL
     ///
     /// ```rust, ignore
@@ -576,7 +655,7 @@ impl<'a> Builder<'a, WithContainerId, ()> {
     pub async fn open(
         mut self,
         url: impl TryInto<Url, Error = url::ParseError>,
-    ) -> Result<ConnectionHandle, OpenError> {
+    ) -> Result<ConnectionHandle<()>, OpenError> {
         let url: Url = url.try_into()?;
 
         // Url info will override the builder fields
@@ -589,25 +668,25 @@ impl<'a> Builder<'a, WithContainerId, ()> {
 
         let addr = url.socket_addrs(|| Some(fe2o3_amqp_types::definitions::PORT))?;
         let stream = TcpStream::connect(&*addr).await?; // std::io::Error
-        
+
         self.open_with_stream(stream).await
     }
 
     /// Open with an IO that implements `AsyncRead` and `AsyncWrite`
-    /// 
+    ///
     /// # TLS
-    /// 
+    ///
     /// If the `scheme` field is `"amqps"`, the builder will attempt to start with
     /// exchanging TLS protocol header.
-    /// 
+    ///
     /// # Alternative TLS establishment
-    /// 
+    ///
     /// This can be used for alternative connection establishment over a TLS stream
-    /// **without** exchanging the TLS protocol header (['A', 'M', 'Q', 'P', 1, 2, 0, 0]).
-    /// 
-    /// An example of establishing connection on a `tokio_native_tls::TlsStream` is shown below. 
+    /// **without** exchanging the TLS protocol header (['A', 'M', 'Q', 'P', 2, 1, 0, 0]).
+    ///
+    /// An example of establishing connection on a `tokio_native_tls::TlsStream` is shown below.
     /// The `tls_stream` can be replaced with a `tokio_rustls::client::TlsStream`.
-    /// 
+    ///
     /// ```rust,ignore
     /// let addr = "localhost:5671";
     /// let domain = "localhost";
@@ -615,7 +694,7 @@ impl<'a> Builder<'a, WithContainerId, ()> {
     /// let connector = native_tls::TlsConnector::new();
     /// let connector = tokio_native_tls::TlsConnector::from(connector);
     /// let tls_stream = connector.connect(domain, stream).await.unwrap();
-    /// 
+    ///
     /// let mut connection = Connection::builder()
     ///     .container_id("connection-1")
     ///     .scheme("amqp")
@@ -628,10 +707,7 @@ impl<'a> Builder<'a, WithContainerId, ()> {
     ///     .unwrap();
     /// ```
     #[allow(unreachable_code)]
-    pub async fn open_with_stream<Io>(
-        self,
-        stream: Io,
-    ) -> Result<ConnectionHandle, OpenError>
+    pub async fn open_with_stream<Io>(self, stream: Io) -> Result<ConnectionHandle<()>, OpenError>
     where
         Io: AsyncRead + AsyncWrite + std::fmt::Debug + Send + Unpin + 'static,
     {
@@ -641,13 +717,15 @@ impl<'a> Builder<'a, WithContainerId, ()> {
                 #[cfg(all(feature = "rustls", not(feature = "native-tls")))]
                 {
                     let domain = self.domain.ok_or_else(|| OpenError::InvalidDomain)?;
-                    return self.connect_tls_with_rustls_default(stream, domain).await
+                    return self.connect_tls_with_rustls_default(stream, domain).await;
                 }
 
                 #[cfg(all(feature = "native-tls", not(feature = "rustls")))]
                 {
                     let domain = self.domain.ok_or_else(|| OpenError::InvalidDomain)?;
-                    return self.connect_tls_with_native_tls_default(stream, domain).await
+                    return self
+                        .connect_tls_with_native_tls_default(stream, domain)
+                        .await;
                 }
 
                 Err(OpenError::TlsConnectorNotFound)
@@ -657,12 +735,16 @@ impl<'a> Builder<'a, WithContainerId, ()> {
     }
 
     #[cfg(all(feature = "rustls", not(feature = "native-tls")))]
-    async fn connect_tls_with_rustls_default<Io>(self, stream: Io, domain: &str) -> Result<ConnectionHandle, OpenError> 
+    async fn connect_tls_with_rustls_default<Io>(
+        self,
+        stream: Io,
+        domain: &str,
+    ) -> Result<ConnectionHandle, OpenError>
     where
         Io: AsyncRead + AsyncWrite + std::fmt::Debug + Send + Unpin + 'static,
     {
+        use librustls::{ClientConfig, OwnedTrustAnchor, RootCertStore};
         use std::sync::Arc;
-        use librustls::{ClientConfig, RootCertStore, OwnedTrustAnchor};
         use tokio_rustls::TlsConnector;
 
         let mut root_cert_store = RootCertStore::empty();
@@ -678,33 +760,33 @@ impl<'a> Builder<'a, WithContainerId, ()> {
         let config = ClientConfig::builder()
             .with_safe_defaults()
             .with_root_certificates(root_cert_store)
-            .with_no_client_auth(); 
+            .with_no_client_auth();
         let connector = TlsConnector::from(Arc::new(config));
         let tls_stream = Transport::connect_tls_with_rustls(stream, domain, &connector).await?;
         self.connect_with_stream(tls_stream).await
     }
-    
+
     #[cfg(all(feature = "native-tls", not(feature = "rustls")))]
-    async fn connect_tls_with_native_tls_default<Io>(self, stream: Io, domain: &str) -> Result<ConnectionHandle, OpenError> 
+    async fn connect_tls_with_native_tls_default<Io>(
+        self,
+        stream: Io,
+        domain: &str,
+    ) -> Result<ConnectionHandle, OpenError>
     where
         Io: AsyncRead + AsyncWrite + std::fmt::Debug + Send + Unpin + 'static,
     {
         use std::io;
 
         let connector = libnative_tls::TlsConnector::new()
-            .map_err(|e| OpenError::Io(io::Error::new(
-                io::ErrorKind::Other,
-                format!("{:?}", e)
-            )))?;
+            .map_err(|e| OpenError::Io(io::Error::new(io::ErrorKind::Other, format!("{:?}", e))))?;
         let connector = tokio_native_tls::TlsConnector::from(connector);
         let tls_stream = Transport::connect_tls_with_native_tls(stream, domain, &connector).await?;
         self.connect_with_stream(tls_stream).await
     }
 }
 
-
-#[cfg(all(feature = "rustls", not(feature = "native-tls")))]
-impl<'a> Builder<'a, WithContainerId, tokio_rustls::TlsConnector> {
+#[cfg(all(feature = "rustls"))]
+impl<'a> Builder<'a, mode::ConnectorWithId, tokio_rustls::TlsConnector> {
     /// Open a [`crate::Connection`] with an url
     ///
     /// # Raw AMQP connection
@@ -718,31 +800,31 @@ impl<'a> Builder<'a, WithContainerId, tokio_rustls::TlsConnector> {
     ///
     /// # TLS
     ///
-    /// TLS is supported with either `tokio-rustls` or `tokio-native-tls` by choosing the 
+    /// TLS is supported with either `tokio-rustls` or `tokio-native-tls` by choosing the
     /// corresponding feature flag.
     ///
     /// ## TLS with feature `"rustls"` enabled
-    /// 
+    ///
     /// ```rust, ignore
     /// let config = rustls::ClientConfig::builder()
     ///     .with_safe_defaults()
     ///     .with_root_certificates(root_cert_store)
     ///     .with_no_client_auth(); // i guess this was previously the default?
     /// let connector = TlsConnector::from(Arc::new(config));
-    /// 
+    ///
     /// let connection = Connection::builder()
     ///     .container_id("connection-1")
     ///     .tls_connector(connector)
     ///     .open("amqps://guest:guest@localhost:5671")
     ///     .await.unwrap();
     /// ```
-    /// 
+    ///
     /// ## TLS with feature `"native-tls"` enabled
-    /// 
+    ///
     /// ```rust,ignore
     /// let cx = native_tls::TlsConnector::new();
     /// let connector = tokio_native_tls::TlsConnector::from(cx);
-    /// 
+    ///
     /// let connection = Connection::builder()
     ///     .container_id("connection-1")
     ///     .tls_connector(connector)
@@ -773,7 +855,7 @@ impl<'a> Builder<'a, WithContainerId, tokio_rustls::TlsConnector> {
     pub async fn open(
         mut self,
         url: impl TryInto<Url, Error = url::ParseError>,
-    ) -> Result<ConnectionHandle, OpenError> {
+    ) -> Result<ConnectionHandle<()>, OpenError> {
         let url: Url = url.try_into()?;
 
         // Url info will override the builder fields
@@ -786,21 +868,18 @@ impl<'a> Builder<'a, WithContainerId, tokio_rustls::TlsConnector> {
 
         let addr = url.socket_addrs(|| Some(fe2o3_amqp_types::definitions::PORT))?;
         let stream = TcpStream::connect(&*addr).await?; // std::io::Error
-        
+
         self.open_with_stream(stream).await
     }
 
     /// Open with an IO that implements `AsyncRead` and `AsyncWrite`
-    /// 
+    ///
     /// # TLS
-    /// 
+    ///
     /// If the `scheme` field is `"amqps"`, the builder will attempt to start with
-    /// exchanging TLS protocol header and establish TLS stream using the user-supplied 
+    /// exchanging TLS protocol header and establish TLS stream using the user-supplied
     /// `tokio_rustls::TlsConnector`.
-    pub async fn open_with_stream<Io>(
-        self,
-        stream: Io,
-    ) -> Result<ConnectionHandle, OpenError>
+    pub async fn open_with_stream<Io>(self, stream: Io) -> Result<ConnectionHandle<()>, OpenError>
     where
         Io: AsyncRead + AsyncWrite + std::fmt::Debug + Send + Unpin + 'static,
     {
@@ -808,7 +887,8 @@ impl<'a> Builder<'a, WithContainerId, tokio_rustls::TlsConnector> {
             "amqp" => self.connect_with_stream(stream).await,
             "amqps" => {
                 let domain = self.domain.ok_or_else(|| OpenError::InvalidDomain)?;
-                let tls_stream = Transport::connect_tls_with_rustls(stream, domain, &self.tls_connector).await?;
+                let tls_stream =
+                    Transport::connect_tls_with_rustls(stream, domain, &self.tls_connector).await?;
                 self.connect_with_stream(tls_stream).await
             }
             _ => Err(OpenError::InvalidScheme),
@@ -816,9 +896,8 @@ impl<'a> Builder<'a, WithContainerId, tokio_rustls::TlsConnector> {
     }
 }
 
-
-#[cfg(all(feature = "native-tls", not(feature = "rustls")))]
-impl<'a> Builder<'a, WithContainerId, tokio_native_tls::TlsConnector> {
+#[cfg(all(feature = "native-tls"))]
+impl<'a> Builder<'a, mode::ConnectorWithId, tokio_native_tls::TlsConnector> {
     /// Open a [`crate::Connection`] with an url
     ///
     /// # Raw AMQP connection
@@ -832,31 +911,31 @@ impl<'a> Builder<'a, WithContainerId, tokio_native_tls::TlsConnector> {
     ///
     /// # TLS
     ///
-    /// TLS is supported with either `tokio-rustls` or `tokio-native-tls` by choosing the 
+    /// TLS is supported with either `tokio-rustls` or `tokio-native-tls` by choosing the
     /// corresponding feature flag.
     ///
     /// ## TLS with feature `"rustls"` enabled
-    /// 
+    ///
     /// ```rust, ignore
     /// let config = rustls::ClientConfig::builder()
     ///     .with_safe_defaults()
     ///     .with_root_certificates(root_cert_store)
     ///     .with_no_client_auth(); // i guess this was previously the default?
     /// let connector = TlsConnector::from(Arc::new(config));
-    /// 
+    ///
     /// let connection = Connection::builder()
     ///     .container_id("connection-1")
     ///     .tls_connector(connector)
     ///     .open("amqps://guest:guest@localhost:5671")
     ///     .await.unwrap();
     /// ```
-    /// 
+    ///
     /// ## TLS with feature `"native-tls"` enabled
-    /// 
+    ///
     /// ```rust,ignore
     /// let cx = native_tls::TlsConnector::new();
     /// let connector = tokio_native_tls::TlsConnector::from(cx);
-    /// 
+    ///
     /// let connection = Connection::builder()
     ///     .container_id("connection-1")
     ///     .tls_connector(connector)
@@ -887,7 +966,7 @@ impl<'a> Builder<'a, WithContainerId, tokio_native_tls::TlsConnector> {
     pub async fn open(
         mut self,
         url: impl TryInto<Url, Error = url::ParseError>,
-    ) -> Result<ConnectionHandle, OpenError> {
+    ) -> Result<ConnectionHandle<()>, OpenError> {
         let url: Url = url.try_into()?;
 
         // Url info will override the builder fields
@@ -900,21 +979,18 @@ impl<'a> Builder<'a, WithContainerId, tokio_native_tls::TlsConnector> {
 
         let addr = url.socket_addrs(|| Some(fe2o3_amqp_types::definitions::PORT))?;
         let stream = TcpStream::connect(&*addr).await?; // std::io::Error
-        
+
         self.open_with_stream(stream).await
     }
 
     /// Open with an IO that implements `AsyncRead` and `AsyncWrite`
-    /// 
+    ///
     /// # TLS
-    /// 
+    ///
     /// If the `scheme` field is `"amqps"`, the builder will attempt to start with
-    /// exchanging TLS protocol header and establish TLS stream using the user-supplied 
+    /// exchanging TLS protocol header and establish TLS stream using the user-supplied
     /// `tokio_rustls::TlsConnector`.
-    pub async fn open_with_stream<Io>(
-        self,
-        stream: Io,
-    ) -> Result<ConnectionHandle, OpenError>
+    pub async fn open_with_stream<Io>(self, stream: Io) -> Result<ConnectionHandle<()>, OpenError>
     where
         Io: AsyncRead + AsyncWrite + std::fmt::Debug + Send + Unpin + 'static,
     {
@@ -922,7 +998,9 @@ impl<'a> Builder<'a, WithContainerId, tokio_native_tls::TlsConnector> {
             "amqp" => self.connect_with_stream(stream).await,
             "amqps" => {
                 let domain = self.domain.ok_or_else(|| OpenError::InvalidDomain)?;
-                let tls_stream = Transport::connect_tls_with_native_tls(stream, domain, &self.tls_connector).await?;
+                let tls_stream =
+                    Transport::connect_tls_with_native_tls(stream, domain, &self.tls_connector)
+                        .await?;
                 self.connect_with_stream(tls_stream).await
             }
             _ => Err(OpenError::InvalidScheme),
