@@ -63,8 +63,7 @@ pub struct ConnectionHandle<R> {
 
 impl<R> std::fmt::Debug for ConnectionHandle<R> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        f.debug_struct("ConnectionHandle")
-            .finish()
+        f.debug_struct("ConnectionHandle").finish()
     }
 }
 
@@ -150,6 +149,11 @@ impl<R> ConnectionHandle<R> {
     }
 }
 
+pub(crate) async fn deallocate_session(control: &mut Sender<ConnectionControl>, session_id: usize) -> Result<(), DeallcoSessionError> {
+    control.send(ConnectionControl::DeallocateSession(session_id)).await
+        .map_err(|_| DeallcoSessionError::IllegalState)
+}
+
 /// An AMQP 1.0 Connection.
 ///
 /// # Open a new [`Connection`] with default configuration
