@@ -36,13 +36,13 @@ pub(crate) mod engine;
 pub(crate) mod frame;
 
 mod error;
-pub(crate) use error::AllocLinkError;
+pub(crate) use error::{AllocLinkError, DeallocLinkError};
 pub use error::Error;
 
 mod builder;
 pub use builder::*;
 
-use self::frame::{SessionFrame, SessionFrameBody};
+use self::{frame::{SessionFrame, SessionFrameBody}};
 
 /// Default incoming_window and outgoing_window
 pub const DEFAULT_WINDOW: UInt = 2048;
@@ -62,8 +62,7 @@ pub struct SessionHandle<R> {
 
 impl<R> std::fmt::Debug for SessionHandle<R> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        f.debug_struct("SessionHandle")
-            .finish()
+        f.debug_struct("SessionHandle").finish()
     }
 }
 
@@ -161,6 +160,15 @@ pub(crate) async fn allocate_link(
     result
 }
 
+pub(crate) async fn deallocate_link(
+    control: &mut mpsc::Sender<SessionControl>,
+    link_name: String,
+) -> Result<(), DeallocLinkError> 
+{
+    control.send(SessionControl::DeallocateLink(link_name)).await
+        .map_err(|_| DeallocLinkError::IllegalState)
+}
+
 /// AMQP1.0 Session
 ///
 /// # Begin a new Session with default configuration
